==> anonnet-0.0.1-20011112.tgz: anonnet/src/thread.c <==
/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

#include "common.h"
#include "log.h"
#include "crypt.h"
#include "thread.h"


void thread_init(void *thr,void (*main)(void *),void *arg) {

	/* initialize members */
	((thread_t *)thr)->__thr_main	= main;
	((thread_t *)thr)->__thr_arg	= (arg == NULL)? thr : arg;

	/* initialize pthread */
	pthread_attr_init(&((thread_t *)thr)->__pthread_attr);

	if (pthread_attr_setdetachstate(&((thread_t *)thr)->__pthread_attr,PTHREAD_CREATE_DETACHED) != 0)
		ERROR("Could not set detached attribute for new thread.");

	((thread_t *)thr)->__alive = false;

	return; /* void */

} /* end thread_init */

void thread_destroy(void *thr) {

	DEBUG("entered thread_destroy");

	pthread_attr_destroy(&((thread_t *)thr)->__pthread_attr);

	return; /* void */

} /* end thread_destroy */


void __thread_destroy(void *thr) {

	DEBUG("entered __thread_destroy");

	thread_destroy((thread_t *)thr);
	object_destroy((object_t *)thr);
	
	return; /* void */
	
} /* end __thread_destroy */


thread_t *thread_create(void (*main)(void *),void *arg) {
	thread_t *thr;
	
	if ((thr = malloc(sizeof(thread_t))) == NULL)
		return NULL;

	object_init((object_t *)thr,__thread_destroy,(void *)thr); /* initialize internal object */
	thread_init(thr,main,arg);

	return thr;

} /* end thread_create */


void thread_free(thread_t *thr) {

	DEBUG("entered thread_free");

	thread_destroy(thr);
	
	free(thr);
	
	return; /* void */

} /* end thread_free */


void thread_run(void *thr) {

	__thread_pre_exec(((thread_t *)thr));

	if (pthread_create(&((thread_t *)thr)->__pthread,&((thread_t *)thr)->__pthread_attr,__thread_run,thr) != 0) {
	
		WARN("Could not run node.");
	
	} else {
		/* something */
	}

	return; /* void */

} /* end thread_run */


bool thread_is_alive(void *thr) {

	return ((thread_t *)thr)->__alive;

} /* end thread_is_alive */


void *__thread_run(void *thr) {

	/*
	 * initialize rand_get for this thread.
	 * it can't be done in __pre_exec because its run in
	 * a creating thread.
	 *
	 */
	rand_init(); /* initialize rand_get for this thread */

	((thread_t *)thr)->__thr_main(((thread_t *)thr)->__thr_arg);

	__thread_post_exec((thread_t *)thr);
	
	return NULL;

} /* end __thread_run */


void __thread_pre_exec(thread_t *thr) {

	object_retain(thr);
	thr->__alive = true;

	return; /* void */

} /* end __thread_pre_exec */


void __thread_post_exec(thread_t *thr) {

	thr->__alive = false;
	object_release(thr);

	DEBUG("Thread ending.");

	return; /* void */

} /* end __thread_post_exec */

==> anonnet-0.0.1-20011210.tgz: anonnet/src/thread.c <==
/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

#include "common.h"
#include "log.h"
#include "crypt.h"
#include "thread.h"


void thread_init(void *thr,void (*main)(void *),void *arg) {

	/* initialize members */
	((thread_t *)thr)->__thr_main	= main;
	((thread_t *)thr)->__thr_arg	= (arg == NULL)? thr : arg;

	/* initialize pthread */
	pthread_attr_init(&((thread_t *)thr)->__pthread_attr);

	if (pthread_attr_setdetachstate(&((thread_t *)thr)->__pthread_attr,PTHREAD_CREATE_DETACHED) != 0)
		ERROR("Could not set detached attribute for new thread.");

	((thread_t *)thr)->__alive = false;

	((thread_t *)thr)->thread_id	= (long int)thr;

	return; /* void */

} /* end thread_init */

void thread_destroy(void *thr) {

	DUMP("entered thread_destroy");

	pthread_attr_destroy(&((thread_t *)thr)->__pthread_attr);

	return; /* void */

} /* end thread_destroy */


thread_t *thread_create(void (*main)(void *),void *arg) {
	thread_t *thr;
	
	if ((thr = _malloc(sizeof(thread_t))) == NULL)
		return NULL;

	object_init((object_t *)thr,thread_free,(void *)thr); /* initialize internal object */
	thread_init(thr,main,arg);

	__objects_created++;

	return thr;

} /* end thread_create */


void thread_free(void *thr) {


	object_destroy(thr);

	DUMP("freeing object %x",((object_t *)thr)->object_id);

	thread_destroy(thr);
	
	_free(thr);

	__objects_freed++;
	
	return; /* void */

} /* end thread_free */


void thread_run(void *thr) {

	__thread_pre_exec(((thread_t *)thr));

	if (pthread_create(&((thread_t *)thr)->__pthread,&((thread_t *)thr)->__pthread_attr,__thread_run,thr) != 0) {
	
		WARN("Could not run node.");
	
	} else {
		/* something */
	}

	return; /* void */

} /* end thread_run */


bool thread_is_alive(void *thr) {

	return ((thread_t *)thr)->__alive;

} /* end thread_is_alive */


void *__thread_run(void *thr) {

	/*
	 * initialize rand_get for this thread.
	 * it can't be done in __pre_exec because its run in
	 * a creating thread.
	 *
	 */
	// rand_get is not thread specific anymore...
	//rand_init(); /* initialize rand_get for this thread */

	((thread_t *)thr)->__thr_main(((thread_t *)thr)->__thr_arg);

	__thread_post_exec((thread_t *)thr);
	
	return NULL;

} /* end __thread_run */


void __thread_pre_exec(thread_t *thr) {

	object_retain(thr);
	thr->__alive = true;

	DUMP("started thread %x",thr->thread_id);

	return; /* void */

} /* end __thread_pre_exec */


void __thread_post_exec(thread_t *thr) {

	DUMP("ending thread %x",thr->thread_id);
	thr->__alive = false;
	object_release(thr);

	return; /* void */

} /* end __thread_post_exec */
